﻿// including the GSM library
#include <GSM.h>
#include <TinyGPS++.h>
#include <AltSoftSerial.h>



//GPS CONFIG
// Choose two Arduino pins to use for software serial
// The GPS Shield uses D2 and D3 by default when in DLINE mode
int RXPin = 14;
int TXPin = 15;

// The Skytaq EM-506 GPS module included in the GPS Shield Kit
// uses 4800 baud by default
int GPSBaud = 4800;

// Create a TinyGPS++ object called "gps"
TinyGPSPlus gps;

// Create a software serial port called "gpsSerial"
AltSoftSerial gpsSerial(RXPin, TXPin);

// PIN number si nécessaire
#define PINNUMBER "0000"

// Parametres APN obtenus de l'opérateur
#define GPRS_APN       "orange" // replace with your GPRS APN
#define GPRS_LOGIN     "orange"    // replace with your GPRS login
#define GPRS_PASSWORD  "orange" // replace with your GPRS password

// ID Arduino
String ID = "3";

// Instanciation des librairies
GSMClient client;
GPRS gprs;
GSM gsmAccess=true; //Si true, affichage des commandes AT

char inString[32]; // string for incoming serial data
int stringPos = 0; // string index counter
boolean startRead = false; // is reading?

// La requête sur "teneur.polytech-lille.net/savedb.php"
char server[] = "teneur.polytech-lille.net"; // URL de base
String meastime = "2015-03-19%2014:56:00"; //date de la mesure
String lat = "56.541"; //latitude
String lon = "7.5412"; //longitude
String measure = "12"; //mesure
String path = "http://teneur.polytech-lille.net/savedb.php?lat=" + lat + "&lon=" + lon + "&measure=" + measure + "&AID=" + ID; //chemin complet de la requête
String gpsconf = "http://teneur.polytech-lille.net/gps_conf.php?ID=" + ID; //Adresse de récupération de l'état du GPS
String timeconf = "http://teneur.polytech-lille.net/time_conf.php?ID=" + ID; //Adresse de récupération de l'intervalle de temps entre mesures
int port = 80; // port, 80 pour HTTP
int PIN_OK = 0;
int SEND_OK = 0;
int GPSEnable = 1;
int timeInt = 5;

void setup()
{
  // On initialise la liaison série à 9600 bauds
  Serial.begin(9600);
  Serial3.begin(GPSBaud);
  Serial.println("Demarrage du client web.");
}

void loop()
{
  while (Serial3.available() > 0)
    if (gps.encode(Serial3.read()))
      fetchData();
    
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {
    Serial.println(F("No GPS detected"));
    while(true);
  }
 
  if (/*SEND_OK*/1){
    Serial.print("Path is : ");
    Serial.println(path);
    magicConnect();
    Serial.println("Gathering ALL the things !");
    sendData();
    Serial.print("I got this : | GPS State : " );
    Serial.print((String)GPSEnable);
    Serial.print(" | Time Interval : ");
    Serial.println((String)timeInt);
    deconnecter();
  }
  else{
     Serial.println("GPS is not ready !");
  }
  
}


void deconnecter(){
  // si le serveur a déco on déconnecte 
  if (!client.available() && !client.connected())
  {
    Serial.println();
    Serial.println("disconnecting.");
    client.stop();

    // do nothing forevermore:
    for(;;);
  }
}

void fetchData(){
      delay(500);   
      // Récupérer la position géographique
      if (gps.date.isValid()){ 
        if (gps.time.isValid()){
          if (gps.location.isValid()){
            char transLat[7];
            char transLon[7];
            dtostrf(gps.location.lat(),2,4,transLat);
            dtostrf(gps.location.lng(),2,4,transLon);
            lat = (String)transLat;
            lon = (String)transLon;
            path = "http://teneur.polytech-lille.net/savedb.php?lat=" + lat + "&lon=" + lon + "&measure=" + measure + "&AID=" + ID;
            SEND_OK = 1;  // SI mise à 1 ici, on continue à utiliser une position factice
          }
          else{
              Serial.print(F("POSITION INVALIDE\n"));
          }
        }
        else{
            Serial.print(F("HEURE INVALIDE\n"));
        }
      }
      else{
        Serial.print(F("DATE INVALIDE\n"));
      }
}


void magicConnect(){
   // etat connexion
  boolean notConnected = true;
  
  // Démarrage du shield GSM
  // On passe le code PIN de la SIM en parametre de gsmAccess.begin()
  while(notConnected){
    if(gsmAccess.begin(PINNUMBER)==GSM_READY && !PIN_OK){
       Serial.println("PIN OK");
       PIN_OK = 1;
       digitalWrite(4,HIGH);
    }
    else if (gprs.attachGPRS(GPRS_APN, GPRS_LOGIN, GPRS_PASSWORD)==GPRS_READY){
            Serial.println("GPRS OK");
            notConnected = false;
            digitalWrite(5,HIGH);
          }
    else{
      Serial.println("Not connected");
      delay(1000);
    }
  }
}

void sendData(){
  Serial.println("connecting...");
  
  // Si on a reussi la connexion au serveur, on execute la requête :
  if (client.connect(server, port)){
    Serial.println("connected");
    // Make a HTTP request:
    client.print("GET ");
    client.print(path);
    client.println(" HTTP/1.0");
    client.println("Connection : keep-alive");
    client.println();
    client.stop();
  }

  if (client.connect(server, port)){  
    client.print("GET ");
    client.print(gpsconf);
    client.println(" HTTP/1.0");
    client.println("Connection : keep-alive");
    client.println();
    GPSEnable = getGPSConf();
    client.stop();
  }

  if (client.connect(server, port)){
    client.print("GET ");
    client.print(timeconf);
    client.println(" HTTP/1.0");
    client.println("Connection : close");
    client.println();
    timeInt = getTimeConf().toInt();  
    client.stop();
  }
}

 
int getGPSConf(){

  Serial.println("Getting GPS conf...");  
 
  //read the page, and capture & return everything between '<' and '>'
  stringPos = 0;
  memset( &inString, 0, 32 ); //clear inString memory

    while(true){    
      if (client.available()) {
        char c = client.read();
        if (c == '<' ){ //'<' is our begining character
          startRead = true; //Ready to start reading the part 
        }
        else if(startRead){
          if(c != '>'){ //'>' is our ending character
            inString[stringPos] = c;
            stringPos ++;
          }
          else{
            //got what we need here! We can disconnect now
            startRead = false;
            //client.stop();client.stop();
            //client.flush();
            //Serial.println("disconnecting.");
            if (inString == "0") return 0;
            else return 1;
          }
        }
      }
    }
}


String getTimeConf(){

  Serial.println("Getting Time configuration...");  
 
  //read the page, and capture & return everything between '<' and '>'
  stringPos = 0;
  memset( &inString, 0, 32 ); //clear inString memory

    while(true){    
      if (client.available()) {
        char c = client.read();
        if (c == '<' ){ //'<' is our begining character
          startRead = true; //Ready to start reading the part 
        }
        else if(startRead){
          if(c != '>'){ //'>' is our ending character
            inString[stringPos] = c;
            stringPos ++;
          }
          else{
            //got what we need here! We can disconnect now
            startRead = false;
            //client.stop();
            //client.flush();
            //Serial.println("disconnecting.");
            return inString;
          }
        }
      }
    }
}
  